#!/bin/bash 
# projectid: 51520
# date: 2024-01-03

script_name=$(basename $0)
PWD=/etc/cpeagent/upgrade/${script_name%.*}

ret_success=0
ret_fail=7
ret_need_upgrade=1

upgrade_path=/etc/cpeagent/upgrade/${script_name%.*}
logfile="/var/log/update.log"
result_id=""
conf_file="/etc/cpeagent/cpeagent.conf"
bak_conf_file="/etc/cpeagent/cpeagent.conf.upgradebak"
softlockup_panic_file="/proc/sys/kernel/softlockup_panic"
panic_on_oops_file="/proc/sys/kernel/panic_on_oops"
panic_on_rcu_stall_file="/proc/sys/kernel/panic_on_rcu_stall"
watchdog_file_path="/sys/class/leds/watchdog-en/brightness"
cpuidle_file_path="/sys/devices/system/cpu/cpu0/cpuidle"
cpufreq_file_path="/sys/kernel/debug/opp/opp_summary"

log()
{
    echo `date` "upgrade-patch:" $@ >> $logfile
}

check_if_need_upgrade()
{
    # check cpu idle
    if [ -e "$cpuidle_file_path" ]; then
        log "cpuidle not disable, need upgrade"
        return $ret_need_upgrade
    else
        log "cpuidle already disabled"
    fi

    #check cpu frequency
    if [ -e "$cpufreq_file_path" ]; then
        if grep -q "1416000000" "$cpufreq_file_path"; then
            log "cpu frequency already modified"
        else
            log "cpu frequency not modified, need upgrade"
            return $ret_need_upgrade
        fi
    fi

    #check panic
    files=("$softlockup_panic_file" "$panic_on_oops_file" "$panic_on_rcu_stall_file")
    for file in "${files[@]}"; do
        result=$(cat "$file")
        if [ "$result" -eq 1 ]; then
            log "File $file has been set to 1"
        else
            log "File $file has not been set to 1, need upgrade"
            return $ret_need_upgrade
        fi
    done

    #check watchdog
    watchdog_value=$(cat $watchdog_file_path)
    if [ "$watchdog_value" -eq 1 ]; then
        log "the watchdog has been enabled"
    else
        log "the watchdog has been enabled,need upgrade"
        return $ret_need_upgrade
    fi
}

update_result_id()
{
    sed -i "/upgradeID/d" $bak_conf_file
    if [ $? -eq 0 ]; then
        log "del upgrade id in conf file succeeded"
    else
        log "del upgrade id in conf file failed"
        exit
    fi

    echo "upgradeID=$1"  >> $bak_conf_file
    if [ $? -eq 0 ]; then
        log "echo upgradeid $1 to conf file succeeded"
    else
        log "echo upgradeid $1 to conf file failed"
        exit
    fi

    cp $bak_conf_file $conf_file -f
    if [ $? -eq 0 ]; then
        log "generate new conf file succeeded"
    else
        log "generate new conf file"
        exit
    fi
}

upgrade_patch()
{
    # 1.cp beforestart.sh & modify upgrade id
    cp ${PWD}/file/beforestart.sh /etc/appex/ -f
    ret=$?
    if [ $ret -eq 0 ]; then
        log "cp beforestart.sh succeed"
    else
        log "cp beforestart.sh failed"
    fi

    sed -i "16s/patch_id=/patch_id=$1/g" /etc/appex/beforestart.sh
    ret=$?
    if [ $ret -eq 0 ]; then
        log "modify upgradeID in beforestart.sh succeed"
    else
        log "modify upgradeID in beforestart.sh failed"
        exit
    fi

    # 2.get userdata dev
    userdata_dev=$(readlink -f /dev/disk/by-partlabel/userdata)
    log "userdata dev:${userdata_dev}"

    # 3. make userdata file
    mkdir -p /userdata
    ret=$?
    if [ $ret -eq 0 ]; then
        log "make userdata file succeed"
    else
        log "make userdata file failed"
        exit
    fi

    # 4. mount
    umount /userdata
    ret=$?
    if [ $ret -eq 0 ]; then
        log "umount userdata file to dev succeed"
    else
        log "file already umount"
    fi

    mount $userdata_dev /userdata/
    ret=$?
    if [ $ret -eq 0 ]; then
        log "mount userdata file to dev succeed"
    else
        log "mount userdata file to dev failed"
        exit
    fi

    # 5.copy upgrade file to userdata
    cp ${PWD}/file/update.img /userdata/update.img -f
    ret=$?
    if [ $ret -eq 0 ]; then
        log "copy upgrade file to userdata succeed"
    else
        log "copy upgrade file to userdata failed"
        exit
    fi

    # 6.updata uboot, will automatically reboot
    # we check the upgrade result in beforestart.sh and write the result to cpeagent.conf.
    update ota
}

install()
{    
    update_result_id $1
    upgrade_patch $1
}

uninstall()
{
    exit $ret_success
}

if [ "$1" == "install" ];then

    #check if upgrade id is null
    if [ -z "$2" ]; then
        log "not get result id, can't upgrade, please check"
        exit $ret_fail
    fi

    #if return value is 1,need upgrade
    check_if_need_upgrade
    ret=$?
    if [ $ret -eq 1 ]; then
        install $2
    else
        exit $ret_success
    fi

elif [ "$1" == "uninstall" ]; then
    uninstall
else
    echo  "invalid args" $ret_fail_others
fi