#!/bin/bash

cpuidle_file_path="/sys/devices/system/cpu/cpu0/cpuidle"
cpufreq_file_path="/sys/kernel/debug/opp/opp_summary"
watchdog_file_path="/sys/class/leds/watchdog-en/brightness"
log_file_path="/var/log/update.log"
conf_file="/etc/cpeagent/cpeagent.conf"
bak_conf_file="/etc/cpeagent/cpeagent.conf.upgradebak"
softlockup_panic_file="/proc/sys/kernel/softlockup_panic"
panic_on_oops_file="/proc/sys/kernel/panic_on_oops"
panic_on_rcu_stall_file="/proc/sys/kernel/panic_on_rcu_stall"
ret_need_upgrade_result=1
ret_not_upgrade_result=2
need_upgrade_result=false
#patch_id will upgrade by PATCH_LW2X06_DRIVER-1.0.5.sh
patch_id=

log()
{
    echo `date` "upgrade-beforestart:" $@ >> $log_file_path
}

check_if_need_upgrade_result()
{
    conf_id=$(grep 'upgradeID' /etc/cpeagent/cpeagent.conf | awk -F'=' '{print $2}' | tr -d ' ')
    current_id=$patch_id

    log "patchid:$current_id conf_id:$conf_id"

    if [ "$current_id" == "$conf_id" ]; then
        log "patchid:$current_id conf_id:$conf_id same current patch version is 1.0.5"
        need_upgrade_result=true
    else
        log "current patch version is not 1.0.5"
        need_upgrade_result=false
    fi


}

write_upgrade_result_conf()
{
    if [ "$need_upgrade_result" == false ]; then
        return $ret_not_upgrade_result
    fi

    rm $bak_conf_file -f
    cp $conf_file $bak_conf_file -f
    if [ $? -eq 0 ];then
        sed -i "/upgraderesult/d" $bak_conf_file
        echo "upgraderesult=$1"  >> $bak_conf_file
        cp $bak_conf_file $conf_file -f
    else
        log "write upgrade result to conf failed"
        exit
    fi

    if [ "$1" == "success" ]; then
        echo 1.0.5 > "/etc/cpeagent/PATCH-DRIVER-VERSION"
    fi
}

enable_watchdog()
{
    echo 1 > "$watchdog_file_path"
    result=$?
    if [ $result -eq 0 ]; then
        log "enable watchdog succeed"
    else
        log "enable watchdog failed"
        write_upgrade_result_conf "failure"
        exit
    fi
}

enable_panic()
{
    files=("$softlockup_panic_file" "$panic_on_oops_file" "$panic_on_rcu_stall_file")

    for file in "${files[@]}"; do
        echo 1 > "$file"
        
        result=$(cat "$file")
        if [ "$result" -eq 1 ]; then
            log "File $file set to: $result - Success"
        else
            log "File $file set to: $result - Failed"
            write_upgrade_result_conf "failure"
            exit
        fi
    done
}

check_upgrade_result()
{

    # check cpu idle
    if [ -e "$cpuidle_file_path" ]; then
        log "cpuidle disable failed"
        write_upgrade_result_conf "failure"
        exit
    else
        log "cpuidle disable succeed"
    fi

    #check cpu frequency
    if [ -e "$cpufreq_file_path" ]; then
        if grep -q "1416000000" "$cpufreq_file_path"; then
            log "cpu frequency modify succeed"
        else
            log "cpu frequency modify failed"
            write_upgrade_result_conf "failure"
            exit
        fi
    else
        log "cpu frequency File not found"
        write_upgrade_result_conf "failure"
        exit
    fi

    write_upgrade_result_conf "success"
}

check_if_need_upgrade_result
enable_panic
enable_watchdog
check_upgrade_result